/**************************************************************
 Program: mexpenduloinvertido.c
 Goal: verso cmex de penduloinvertido_simular.m
 Chamada: [mexEstado,F,FimDeCurso] = mexpenduloinvertido(mexEstado,mexParametros,F,T);
 Compilao MATLAB: mex mexpenduloinvertido.c
**************************************************************/
/* No esquecer: x[i][j] (C Format) <=> x[i+j*M] (CMEX Format) */

#include "mex.h"

#include <windows.h>
#include <math.h>

#define MATHCONST_PI	3.14159265358979310000
#define MATHCONST_2PI	6.28318530717958620000
					 
#define RAD_TO_DEG(a)	((a)*180.0/MATHCONST_PI)
#define DEG_TO_RAD(a)	((a)*MATHCONST_PI/180.0)

#define SQR(a)  ((a)*(a))
#define SQRT(a)  (sqrt(a))
#define MAX(a,b)		(((a)>=(b)) ? (a) : (b))
#define MIN(a,b)		(((a)<=(b)) ? (a) : (b))
#define ABS(a)			(((a)<0) ? (-a) : (a))

typedef struct {
	//mexParametros = [m,M,b,l,I,g,xmax,xmin,thetamax,thetamin];
	double m;
	double M;
	double b;
	double l;
	double I;
	double g;
	double xmax;
	double xmin;
	double thetamax;
	double thetamin;
} PENDULOPARAMETROS, *PPENDULOPARAMETROS;

typedef struct {
	// mexEstado = [x,xdot,theta,thetadot];
	double x;
	double xdot;
	double theta;
	double thetadot;
} PENDULOESTADO, *PPENDULOESTADO;

BOOL PenduloDinamica(PPENDULOESTADO pPenduloEstado, PPENDULOPARAMETROS pPenduloParametros, double F, double dt);

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	double  *pData;
	double  F,dt;
	PENDULOPARAMETROS PenduloParametros;
	PENDULOESTADO PenduloEstado;
	BOOL FimDeCurso;

	/* Verificar as entradas */
	if(nrhs!=4){
		mexErrMsgTxt("Sao necessarias 4 variaves de entrada.");
	}
	if(nlhs!=2){
		mexErrMsgTxt("Sao necessarias 2 variaveis de saida.");
	}

	/* Variaveis de entrada */
	// mexEstado = [x,xdot,theta,thetadot];
	pData = mxGetPr(prhs[0]);
	PenduloEstado.x			= pData[0];
	PenduloEstado.xdot		= pData[1];
	PenduloEstado.theta		= pData[2];
	PenduloEstado.thetadot  = pData[3];
	

	//mexParametros = [m,M,b,l,I,g];
	pData = mxGetPr(prhs[1]);
	PenduloParametros.m = pData[0]; 
	PenduloParametros.M = pData[1];
	PenduloParametros.b = pData[2];
	PenduloParametros.l = pData[3];
	PenduloParametros.I = pData[4];
	PenduloParametros.g = pData[5];
	PenduloParametros.xmax = pData[6];
	PenduloParametros.xmin = pData[7];
	PenduloParametros.thetamax = pData[8];
	PenduloParametros.thetamin = pData[9];

	F  = mxGetScalar(prhs[2]);
	dt = mxGetScalar(prhs[3]);

	/* Calculo */
	FimDeCurso = PenduloDinamica(&PenduloEstado,&PenduloParametros,F,dt);
	
	/* Saida */
	plhs[0] = mxCreateDoubleMatrix(4,1,mxREAL);
	*(mxGetPr(plhs[0])+0) = PenduloEstado.x;
	*(mxGetPr(plhs[0])+1) = PenduloEstado.xdot;
	*(mxGetPr(plhs[0])+2) = PenduloEstado.theta;
	*(mxGetPr(plhs[0])+3) = PenduloEstado.thetadot;


	plhs[1] = mxCreateDoubleMatrix(1,1,mxREAL);
	if (FimDeCurso){
		*(mxGetPr(plhs[1])+0) = 0.0;
	}
	else{
		*(mxGetPr(plhs[1])+0) = 1.0;
	}
	//	printf("\n");*/
}

BOOL PenduloDinamica(PPENDULOESTADO pPenduloEstado, PPENDULOPARAMETROS pPenduloParametros, double F, double dt)
{
	int n,N;
	double dtt;
	double x1;
	double x2;
	double x3;
	double x4;
	double dx1dt;
	double dx2dt;
	double dx3dt;
	double dx4dt;
	double b2;
	double b4;
	double lambda;
	double m;
	double M;
	double b;
	double l;
	double I;
	double g;
	BOOL FimDeCurso;

	m = pPenduloParametros->m; 
	M = pPenduloParametros->M;
	b = pPenduloParametros->b;
	l = pPenduloParametros->l;
	I = pPenduloParametros->I;
	g = pPenduloParametros->g;
	
	// Lao principal:
	FimDeCurso = 0;
	N = 20;
	dtt = dt/((double)(N));

	x1 = pPenduloEstado->x;
	x2 = pPenduloEstado->xdot;
	x3 = pPenduloEstado->theta;
	x4 = pPenduloEstado->thetadot;
	for(n=0;n<N;++n){
		lambda = (I+m*l*l)*(M+m) - m*m*l*l*cos(x3)*cos(x3);
		b2 = -m*l*(I+m*l*l)*sin(x3)*(x4*x4) + m*m*l*l*g*sin(x3)*cos(x3) + F*(I+m*l*l) - x2*(I+m*l*l)*b;
		b4 = -(M+m)*m*g*l*sin(x3)+m*l*b*cos(x3)*x2+m*m*l*l*sin(x3)*cos(x3)-m*l*cos(x3)*F;
      
		dx1dt = x2;
		dx2dt = b2/lambda;
		dx3dt = x4;
		dx4dt = b4/lambda;
      
		x1 += dtt*dx1dt;
		x2 += dtt*dx2dt;
		x3 += dtt*dx3dt;
		x4 += dtt*dx4dt;
      
		if (x1 > pPenduloParametros->xmax){
			x1 = pPenduloParametros->xmax;
	        x2 = 0.0;
	        FimDeCurso = 1;
		}
		if (x1 < pPenduloParametros->xmin){
			x1 = pPenduloParametros->xmin;
	        x2 = 0.0;
		    FimDeCurso = 1;
		}
		if (x3 > pPenduloParametros->thetamax){
			x3 = pPenduloParametros->thetamax;
			x4 = 0.0;
		}
		if (x3 < pPenduloParametros->thetamin){
			x3 = pPenduloParametros->thetamin;
			x4 = 0.0;
		}
	}
	pPenduloEstado->x = x1;
	pPenduloEstado->xdot = x2; 
	pPenduloEstado->theta = x3; 
	pPenduloEstado->thetadot = x4; 

	return FimDeCurso;
}

