%%% Arquivo: sim_malhaaberta.m
%%% Autor: G. A. Borges
%%% Objetivo: Simular um pndulo invertido em malha aberta
echo off;
clear all;
close all;
format long;

FlagSalvarFiguras = 1;

% Variveis do tempo:
T = 0.005; 		% perodo de amostragem
t = [0:T:10]';	% tempo de simulao
N = length(t);

% Iniciar modelo
[StrEstado,StrEntrada,StrMedicao,StrParametros] = penduloinvertido_iniciar(N);

% Estado inicial:
StrEstado.x(1) = 0;
StrEstado.theta(1) = pi;
StrEstado.xdot(1) = 0;
StrEstado.thetadot(1) = 0;

StrMedicao.x(1) = StrEstado.x(1);
StrMedicao.theta(1) = StrEstado.theta(1);

% Desenhar:
HandlerPendulo = penduloinvertido_desenhar(StrEstado,StrParametros,1,1);
if FlagSalvarFiguras,
	print penduloinvertido_fig0.png -dpng -f1;
end

% Lao principal
TFimDeCurso = 0; % Controle de fim-de-curso
F = 0; % Entrada de controle
for n=2:N,
   %%% Simulao: x(n) = f(x,u,n-1)
   [StrEstado,StrEntrada,StrMedicao,FimDeCurso] = penduloinvertido_simular(StrEstado,StrEntrada,StrMedicao,StrParametros,F,T,n);
   
   %%% Lei de controle u(n): malha aberta.
   if n<1*N/4
      F = 0.1;
   else
      F = 0.0;
   end
   
   %%% Atualizar desenho:
   if rem(n,fix(0.2/T))==0,
       HandlerPendulo = penduloinvertido_desenhar(StrEstado,StrParametros,n,1,HandlerPendulo);
       drawnow;
   end
   
   %%% Pra a simulao antecipadamente se o pndulo ficar no fim de curso por mais de 3 segundos:
   if FimDeCurso == 1
      TFimDeCurso = TFimDeCurso + T;
   else,
      TFimDeCurso = 0;
	end      
   if FimDeCurso == 1 & TFimDeCurso > 3.0,
      break;
   end
end

%%% Grficos:
figure(2);
plot(t(1:n),(StrEstado.theta(1:n))*180/pi,'r'); hold on;
plot(t(1:n),(StrMedicao.theta(1:n))*180/pi,'b'); hold on;
xlabel('t [s]');
ylabel('\theta [deg]');
ax = axis; axis([ax(1:2) StrParametros.thetamin*180/pi-10 StrParametros.thetamax*180/pi+10]);

figure(3);
plot(t(1:n),(StrEstado.thetadot(1:n))*180/pi,'r'); hold on;
xlabel('t [s]');
ylabel('d{\theta}/dt [deg/s]');

figure(4);
plot(t(1:n),(StrEstado.x(1:n)),'r'); hold on;
plot(t(1:n),(StrMedicao.x(1:n)),'b'); hold on;
xlabel('t [s]');
ylabel('x [m]');
ax = axis; axis([ax(1:2) StrParametros.xmin-0.1 StrParametros.xmax+0.1]);

figure(5);
plot(t(1:n),(StrEstado.xdot(1:n)),'r'); hold on;
xlabel('t [s]');
ylabel('dx/dt [m/s]');

figure(6);
plot(t(1:n),(StrEntrada.u(1:n)),'r'); hold on;
xlabel('t [s]');
ylabel('F [N]');
ax = axis; axis([ax(1:2) ax(3)*1.5-0.1 ax(4)*1.5+0.1]);

%Salva figuras:
if FlagSalvarFiguras,
   print penduloinvertido_fig1.png -dpng -f1;
   print penduloinvertido_fig2.png -dpng -f2;
   print penduloinvertido_fig3.png -dpng -f3;
   print penduloinvertido_fig4.png -dpng -f4;
   print penduloinvertido_fig5.png -dpng -f5;
   print penduloinvertido_fig6.png -dpng -f6;
end